"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var http_1 = require('@angular/http');
var testing_1 = require('@angular/http/testing');
var testing_2 = require('@angular/core/testing');
require('rxjs/Rx');
var api_1 = require('./api');
var config_1 = require('../config');
var TestConfig = (function (_super) {
    __extends(TestConfig, _super);
    function TestConfig() {
        _super.apply(this, arguments);
        this.baseUrl = 'http://www.example.com';
    }
    return TestConfig;
}(config_1.Config));
testing_2.describe('ApiService', function () {
    testing_2.beforeEachProviders(function () { return [
        api_1.ApiService,
        testing_1.MockBackend,
        http_1.BaseRequestOptions,
        {
            provide: http_1.Http,
            useFactory: function (backend, options) { return new http_1.Http(backend, options); },
            deps: [testing_1.MockBackend, http_1.BaseRequestOptions]
        },
        {
            provide: config_1.Config,
            useClass: TestConfig
        }]; });
    function makeResponse(response) {
        return new http_1.Response(new http_1.ResponseOptions(response));
    }
    testing_2.it('Uses the baseUrl from the configuration', testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
        var response = {
            body: JSON.stringify({ id: 666 }),
            status: 200
        };
        mockBackEnd.connections.subscribe(function (connection) {
            testing_2.expect(connection.request.url)
                .toEqual('http://www.example.com/lists/666');
            connection.mockRespond(makeResponse(response));
        });
        return api.fetch('lists/666')
            .subscribe(function (data) { console.log(); });
    })));
    testing_2.it('returns the json when response is 200', testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
        var response = {
            body: JSON.stringify({ id: 666 }),
            status: 200
        };
        mockBackEnd.connections.subscribe(function (connection) {
            connection.mockRespond(makeResponse(response));
        });
        return api.fetch('lists/666').subscribe(function (data) {
            testing_2.expect(data).toEqual({ id: 666 });
        });
    })));
    testing_2.it('throws an error when the body is not valid json', testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
        mockBackEnd.connections
            .subscribe(function (connection) {
            var response = {
                body: '{!"',
                status: 200
            };
            mockBackEnd.connections.subscribe(function (connection) {
                connection.mockRespond(makeResponse(response));
            });
        });
        return api.fetch('lists/666').subscribe(function (data) {
            testing_2.expect(data).toEqual({ id: 666 });
        }, function (err) {
            testing_2.expect(err).toEqual(jasmine.any(Error));
        });
    })));
    [404, 500].forEach(function (statusCode) {
        testing_2.it("throws an error when response is " + statusCode, testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
            mockBackEnd.connections.subscribe(function (connection) {
                connection.mockRespond(new http_1.Response(new http_1.ResponseOptions({
                    status: statusCode
                })));
            });
            return api.fetch('lists/666')
                .subscribe(function (res) {
                throw new Error('This test should not get a successful response');
            }, function (err) {
                testing_2.expect(err).toEqual(jasmine.any(api_1.ApiError));
                testing_2.expect(err.response.status).toEqual(statusCode);
            });
        })));
    });
    testing_2.it('fails gracefully when the API is unavailable', testing_2.async(testing_2.inject([api_1.ApiService, testing_1.MockBackend], function (api, mockBackEnd) {
        mockBackEnd.connections.subscribe(function (connection) {
            connection.mockError(new Error('error'));
        });
        return api.fetch('lists/666')
            .subscribe(function (res) {
            throw new Error('This test should not get a successful response');
        }, function (err) {
            testing_2.expect(err).toEqual(jasmine.any(Error));
        });
    })));
});
//# sourceMappingURL=api.test.js.map